#ifndef BODY_HH
#define BODY_HH
#include "point.hh"
template<class T> class Vector;
class Grafix;
class Vehicle;
class View;
class World;

class Body { protected:

  Point _position;
  int _z_plane;
  double _radius;
  
 public:

  Body();
  
  virtual Body *clone();
  virtual Body *clone_attachment(Body *);
  virtual void embody(World *);
  virtual void place(const Point &, double);
  virtual void reset();
  
  Point position() const		{ return _position; }
  virtual double rotation() const;
  int z_plane() const			{ return _z_plane; }
  double radius() const			{ return _radius; }
  
  virtual bool has_stimulus(int);
  virtual double stimulus(int, Point, Point &);
  virtual void sensation(const Vector<double> &);
  
  virtual double intersects(Point, double angle);
  virtual bool attached_to(const Body *) const;

  bool might_collide(const Body *) const;
  virtual void collide(Body *);
  
  virtual bool movable();
  virtual int move_phase() const;
  virtual Point velocity();
  virtual void move(double time);
  
  virtual void trail_snapshot();
  
  virtual void draw(View *);
  
  virtual Vehicle *cast_vehicle()		{ return 0; }
  
};

inline
Body::Body()
  : _position(0, 0), _z_plane(0), _radius(0)
{
}

inline bool
Body::might_collide(const Body *o) const
{
  if (_z_plane <= 0 || _z_plane != o->_z_plane)
    return false;
  Point p = _position - o->_position;
  return (fabs(p.x) + fabs(p.y) <= _radius + o->_radius);
}

#endif
