#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "moment.hh"
#include <assert.h>
#include <sys/time.h>
#include <math.h>

#ifdef X_GETTIMEOFDAY
#define ET_GETTIMEOFDAY(t) X_GETTIMEOFDAY(t)
#elif GETTIMEOFDAY_PROTO == 0
extern "C" int gettimeofday(struct timeval *, struct timezone *);
#define ET_GETTIMEOFDAY(t) gettimeofday((t), 0)
#elif GETTIMEOFDAY_PROTO == 1
#define ET_GETTIMEOFDAY(t) gettimeofday((t))
#else
#define ET_GETTIMEOFDAY(t) gettimeofday((t), 0)
#endif


Moment Moment::genesis((Moment *)0);


Moment::Moment(Moment *m)
{
  assert(m == 0);
  struct timeval tv;
  ET_GETTIMEOFDAY(&tv);
  _sec = tv.tv_sec;
  _usec = tv.tv_usec;
}

Moment
Moment::now()
{
  Moment nowish((Moment *)0);
  return nowish -= genesis;
}

Moment &
Moment::operator*=(double d)
{
  double real = (double)_sec + (double)_usec / MICRO_PER_SEC;
  
  double new_sec, new_usec;
  new_usec = modf(real * d, &new_sec);
  
  _sec = (long)new_sec;
  _usec = (long)(new_usec * MICRO_PER_SEC);
  return *this;
}

double
operator/(const Moment &a, const Moment &b)
{
  double areal = (double)a._sec * Moment::MICRO_PER_SEC + a._usec;
  double breal = (double)b._sec * Moment::MICRO_PER_SEC + b._usec;
  return areal / breal;
}
